@@ -1,6 +1,6 @@
 /*
   The oSIP library implements the Session Initiation Protocol (SIP -rfc3261-)
-  Copyright (C) 2001,2002,2003,2004  Aymeric MOIZARD jack@atosc.org
+  Copyright (C) 2001,2002,2003,2004,2005,2006,2007 Aymeric MOIZARD jack@atosc.org
   
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -65,7 +65,7 @@ extern "C"
     char *call_id;                       /**< Call-ID*/
     char *local_tag;                     /**< local tag */
     char *remote_tag;                    /**< remote tag */
-    osip_list_t *route_set;              /**< route set */
+    osip_list_t route_set;              /**< route set */
     int local_cseq;                      /**< last local cseq */
     int remote_cseq;                     /**< last remote cseq*/
     osip_to_t *remote_uri;               /**< remote_uri */
@@ -84,7 +84,7 @@ extern "C"
  * @param instance The application instance
  */
 #define osip_dialog_set_instance(dialog,instance) (dialog)->your_instance = (void*)(instance)
-  
+
 /**
  * Retrieve application instance from dialog
  * @param dialog The osip dialog
@@ -103,8 +103,7 @@ extern "C"
  * @param dialog The element to allocate.
  * @param response The response containing the informations.
  */
-  int osip_dialog_init_as_uac (osip_dialog_t ** dialog,
-			       osip_message_t * response);
+  int osip_dialog_init_as_uac (osip_dialog_t ** dialog, osip_message_t * response);
 /**
  * Allocate a osip_dialog_t element as a UAC.
  * <UL><LI>This could be used to initiate dialog with a NOTIFY coming
@@ -114,8 +113,8 @@ extern "C"
  * @param local_cseq The local cseq
  */
   int osip_dialog_init_as_uac_with_remote_request (osip_dialog_t ** dialog,
-						   osip_message_t *next_request,
-						   int local_cseq);
+                                                   osip_message_t *
+                                                   next_request, int local_cseq);
 
 /**
  * Allocate a osip_dialog_t element as a UAS.
@@ -126,8 +125,7 @@ extern "C"
  * @param response The response containing other informations.
  */
   int osip_dialog_init_as_uas (osip_dialog_t ** dialog,
-			       osip_message_t * invite,
-			       osip_message_t * response);
+                               osip_message_t * invite, osip_message_t * response);
 /**
  * Free all resource in a osip_dialog_t element.
  * @param dialog The element to free.
@@ -151,7 +149,7 @@ extern "C"
  * @param invite The invite received.
  */
   int osip_dialog_update_route_set_as_uas (osip_dialog_t * dialog,
-					   osip_message_t * invite);
+                                           osip_message_t * invite);
 /**
  * Update the CSeq (remote cseq) during a UAS transaction of a dialog.
  * NOTE: All INCOMING transactions MUST update the remote CSeq.
@@ -159,15 +157,14 @@ extern "C"
  * @param request The request received.
  */
   int osip_dialog_update_osip_cseq_as_uas (osip_dialog_t * dialog,
-					   osip_message_t * request);
+                                           osip_message_t * request);
 
 /**
  * Match a response received with a dialog.
  * @param dialog The element to work on.
  * @param response The response received.
  */
-  int osip_dialog_match_as_uac (osip_dialog_t * dialog,
-				osip_message_t * response);
+  int osip_dialog_match_as_uac (osip_dialog_t * dialog, osip_message_t * response);
 /**
  * Update the tag as UAC of a dialog?. (this could be needed if the 180
  * does not contains any tag, but the 200 contains one.
@@ -175,7 +172,7 @@ extern "C"
  * @param response The response received.
  */
   int osip_dialog_update_tag_as_uac (osip_dialog_t * dialog,
-				     osip_message_t * response);
+                                     osip_message_t * response);
 /**
  * Update the Route-Set as UAC of a dialog.
  * NOTE: bis-09 says that only INVITE transactions can update the route-set.
@@ -187,15 +184,14 @@ extern "C"
  * @param response The response received.
  */
   int osip_dialog_update_route_set_as_uac (osip_dialog_t * dialog,
-					   osip_message_t * response);
+                                           osip_message_t * response);
 
 /**
- * Match a request (response sent??) received with a dialog.
+ * Match a request (response sent?) received with a dialog.
  * @param dialog The element to work on.
  * @param request The request received.
  */
-  int osip_dialog_match_as_uas (osip_dialog_t * dialog,
-				osip_message_t * request);
+  int osip_dialog_match_as_uas (osip_dialog_t * dialog, osip_message_t * request);
 
 /**
  * @internal
