@@ -1,6 +1,6 @@
 /*
-  The oSIP library implements the Session Initiation Protocol (SIP -rfc2543-)
-  Copyright (C) 2001  Aymeric MOIZARD jack@atosc.org
+  The oSIP library implements the Session Initiation Protocol (SIP -rfc3261-)
+  Copyright (C) 2001,2002,2003,2004,2005,2006,2007 Aymeric MOIZARD jack@atosc.org
   
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -36,7 +36,7 @@
 
 /**
  * Structure for Contact headers.
- * @defvar osip_contact_t
+ * @var osip_contact_t
  */
   typedef osip_from_t osip_contact_t;
 
@@ -45,6 +45,7 @@ extern "C"
 {
 #endif
 
+#ifndef MINISIZE
 /**
  * Allocate a Contact element.
  * @param header The element to work on.
@@ -55,6 +56,7 @@ extern "C"
  * @param header The element to work on.
  */
   void osip_contact_free (osip_contact_t * header);
+#endif
 /**
  * Parse a Contact element.
  * @param header The element to work on.
@@ -67,12 +69,18 @@ extern "C"
  * @param dest A pointer on the new allocated string.
  */
   int osip_contact_to_str (const osip_contact_t * header, char **dest);
+#ifndef MINISIZE
 /**
  * Clone a Contact element.
  * @param header The element to work on.
  * @param dest A pointer on the copy of the element.
  */
   int osip_contact_clone (const osip_contact_t * header, osip_contact_t ** dest);
+#else
+  #define osip_contact_init  osip_from_init
+  #define osip_contact_free  osip_from_free
+  #define osip_contact_clone osip_from_clone
+#endif
 /**
  * Get the displayname from a Contact header.
  * @param header The element to work on.
@@ -108,14 +116,14 @@ extern "C"
  * @param name The token name.
  * @param value The token value.
  */
-#define osip_contact_param_add(header,name, value) osip_generic_param_add((header)->gen_params, name,value)
+#define osip_contact_param_add(header,name, value) osip_generic_param_add((&(header)->gen_params), name,value)
 /**
  * Find a header parameter in a Contact element.
  * @param header The element to work on.
  * @param name The token name to search.
  * @param dest A pointer on the element found.
  */
-#define osip_contact_param_get_byname(header,name,dest) osip_generic_param_get_byname((header)->gen_params,name,dest)
+#define osip_contact_param_get_byname(header,name,dest) osip_generic_param_get_byname((&(header)->gen_params),name,dest)
 
 
 #ifdef __cplusplus
