@@ -1,6 +1,6 @@
 /*
   The oSIP library implements the Session Initiation Protocol (SIP -rfc3261-)
-  Copyright (C) 2001,2002,2003  Aymeric MOIZARD jack@atosc.org
+  Copyright (C) 2001,2002,2003,2004,2005,2006,2007 Aymeric MOIZARD jack@atosc.org
   
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -22,6 +22,8 @@
 
 #include "fsm.h"
 
+#ifndef MINISIZE
+
 osip_statemachine_t *ist_fsm;
 
 osip_statemachine_t *
@@ -36,12 +38,12 @@ __ist_unload_fsm ()
   transition_t *transition;
   osip_statemachine_t *statemachine = __ist_get_fsm ();
 
-  while (!osip_list_eol (statemachine->transitions, 0))
+  for (transition = statemachine->transitions; transition != NULL; transition = statemachine->transitions)
     {
-      transition = (transition_t *) osip_list_get (statemachine->transitions, 0);
-      osip_list_remove (statemachine->transitions, 0);
+      REMOVE_ELEMENT(statemachine->transitions, transition);
       osip_free (transition);
     }
+
   osip_free (statemachine->transitions);
   osip_free (statemachine);
 }
@@ -53,127 +55,161 @@ __ist_load_fsm ()
   transition_t *transition;
 
   ist_fsm = (osip_statemachine_t *) osip_malloc (sizeof (osip_statemachine_t));
-  ist_fsm->transitions = (osip_list_t *) osip_malloc (sizeof (osip_list_t));
-  osip_list_init (ist_fsm->transitions);
+  ist_fsm->transitions = NULL;
 
   transition = (transition_t *) osip_malloc (sizeof (transition_t));
   transition->state = IST_PRE_PROCEEDING;
   transition->type = RCV_REQINVITE;
   transition->method = (void (*)(void *, void *)) &ist_rcv_invite;
-  osip_list_add (ist_fsm->transitions, transition, -1);
+  ADD_ELEMENT (ist_fsm->transitions, transition);
 
   transition = (transition_t *) osip_malloc (sizeof (transition_t));
   transition->state = IST_PROCEEDING;
   transition->type = RCV_REQINVITE;
   transition->method = (void (*)(void *, void *)) &ist_rcv_invite;
-  osip_list_add (ist_fsm->transitions, transition, -1);
+  ADD_ELEMENT (ist_fsm->transitions, transition);
 
   transition = (transition_t *) osip_malloc (sizeof (transition_t));
   transition->state = IST_COMPLETED;
   transition->type = RCV_REQINVITE;
   transition->method = (void (*)(void *, void *)) &ist_rcv_invite;
-  osip_list_add (ist_fsm->transitions, transition, -1);
+  ADD_ELEMENT (ist_fsm->transitions, transition);
 
   transition = (transition_t *) osip_malloc (sizeof (transition_t));
   transition->state = IST_COMPLETED;
   transition->type = TIMEOUT_G;
   transition->method = (void (*)(void *, void *)) &osip_ist_timeout_g_event;
-  osip_list_add (ist_fsm->transitions, transition, -1);
+  ADD_ELEMENT (ist_fsm->transitions, transition);
 
   transition = (transition_t *) osip_malloc (sizeof (transition_t));
   transition->state = IST_COMPLETED;
   transition->type = TIMEOUT_H;
   transition->method = (void (*)(void *, void *)) &osip_ist_timeout_h_event;
-  osip_list_add (ist_fsm->transitions, transition, -1);
+  ADD_ELEMENT (ist_fsm->transitions, transition);
 
   transition = (transition_t *) osip_malloc (sizeof (transition_t));
   transition->state = IST_PROCEEDING;
   transition->type = SND_STATUS_1XX;
   transition->method = (void (*)(void *, void *)) &ist_snd_1xx;
-  osip_list_add (ist_fsm->transitions, transition, -1);
+  ADD_ELEMENT (ist_fsm->transitions, transition);
 
   transition = (transition_t *) osip_malloc (sizeof (transition_t));
   transition->state = IST_PROCEEDING;
   transition->type = SND_STATUS_2XX;
   transition->method = (void (*)(void *, void *)) &ist_snd_2xx;
-  osip_list_add (ist_fsm->transitions, transition, -1);
+  ADD_ELEMENT (ist_fsm->transitions, transition);
 
   transition = (transition_t *) osip_malloc (sizeof (transition_t));
   transition->state = IST_PROCEEDING;
   transition->type = SND_STATUS_3456XX;
   transition->method = (void (*)(void *, void *)) &ist_snd_3456xx;
-  osip_list_add (ist_fsm->transitions, transition, -1);
+  ADD_ELEMENT (ist_fsm->transitions, transition);
 
   transition = (transition_t *) osip_malloc (sizeof (transition_t));
   transition->state = IST_COMPLETED;
   transition->type = RCV_REQACK;
   transition->method = (void (*)(void *, void *)) &ist_rcv_ack;
-  osip_list_add (ist_fsm->transitions, transition, -1);
+  ADD_ELEMENT (ist_fsm->transitions, transition);
 
   transition = (transition_t *) osip_malloc (sizeof (transition_t));
   transition->state = IST_CONFIRMED;
   transition->type = RCV_REQACK;
   transition->method = (void (*)(void *, void *)) &ist_rcv_ack;
-  osip_list_add (ist_fsm->transitions, transition, -1);
+  ADD_ELEMENT (ist_fsm->transitions, transition);
 
   transition = (transition_t *) osip_malloc (sizeof (transition_t));
   transition->state = IST_CONFIRMED;
   transition->type = TIMEOUT_I;
   transition->method = (void (*)(void *, void *)) &osip_ist_timeout_i_event;
-  osip_list_add (ist_fsm->transitions, transition, -1);
+  ADD_ELEMENT (ist_fsm->transitions, transition);
 
 }
 
-osip_message_t *
-ist_create_resp_100 (osip_transaction_t * ist, osip_message_t * request)
-{
-  int i;
-  osip_message_t *resp_100;
+#else
 
-  i = osip_message_init (&resp_100);
-  if (i != 0)
-    return NULL;
-
-  /* follow instructions from 8.2.6 */
-  i = osip_from_clone (request->from, &(resp_100->from));
-  if (i != 0)
-    goto icr_error;
-  /* 17.2.1 says: should NOT add a tag */
-  i = osip_to_clone (request->to, &(resp_100->to));     /* DOES NOT include any tag! */
-  if (i != 0)
-    goto icr_error;
-  i = osip_call_id_clone (request->call_id, &(resp_100->call_id));
-  if (i != 0)
-    goto icr_error;
-  i = osip_cseq_clone (request->cseq, &(resp_100->cseq));
-  if (i != 0)
-    goto icr_error;
-
-  /* Via headers are copied from request */
+transition_t ist_transition[11] =
   {
-    int pos = 0;
-    osip_via_t *via;
-    osip_via_t *orig_via;
+    {
+      IST_PRE_PROCEEDING,
+      RCV_REQINVITE,
+      (void (*)(void *, void *)) &ist_rcv_invite,
+      &ist_transition[1], NULL
+    }
+    ,
+    {
+      IST_PROCEEDING,
+      RCV_REQINVITE,
+      (void (*)(void *, void *)) &ist_rcv_invite,
+      &ist_transition[2], NULL
+    }
+    ,
+    {
+      IST_COMPLETED,
+      RCV_REQINVITE,
+      (void (*)(void *, void *)) &ist_rcv_invite,
+      &ist_transition[3], NULL
+    }
+    ,
+    {
+      IST_COMPLETED,
+      TIMEOUT_G,
+      (void (*)(void *, void *)) &osip_ist_timeout_g_event,
+      &ist_transition[4], NULL
+    }
+    ,
+    {
+      IST_COMPLETED,
+      TIMEOUT_H,
+      (void (*)(void *, void *)) &osip_ist_timeout_h_event,
+      &ist_transition[5], NULL
+    }
+    ,
+    {
+      IST_PROCEEDING,
+      SND_STATUS_1XX,
+      (void (*)(void *, void *)) &ist_snd_1xx,
+      &ist_transition[6], NULL
+    }
+    ,
+    {
+      IST_PROCEEDING,
+      SND_STATUS_2XX,
+      (void (*)(void *, void *)) &ist_snd_2xx,
+      &ist_transition[7], NULL
+    }
+    ,
+    {
+      IST_PROCEEDING,
+      SND_STATUS_3456XX,
+      (void (*)(void *, void *)) &ist_snd_3456xx,
+      &ist_transition[8], NULL
+    }
+    ,
+    {
+      IST_COMPLETED,
+      RCV_REQACK,
+      (void (*)(void *, void *)) &ist_rcv_ack,
+      &ist_transition[9], NULL
+    }
+    ,
+    {
+      IST_CONFIRMED,
+      RCV_REQACK,
+      (void (*)(void *, void *)) &ist_rcv_ack,
+      &ist_transition[10], NULL
+    }
+    ,
+    {
+      IST_CONFIRMED,
+      TIMEOUT_I,
+      (void (*)(void *, void *)) &osip_ist_timeout_i_event,
+      NULL, NULL
+    }
+  };
 
-    while (!osip_list_eol (ist->orig_request->vias, pos))
-      {
-        orig_via = (osip_via_t *) osip_list_get (ist->orig_request->vias, pos);
-        osip_via_clone (orig_via, &via);
-        osip_list_add (resp_100->vias, via, -1);
-        pos++;
-      }
-  }
+osip_statemachine_t ist_fsm = { ist_transition };
 
-  /* TODO: */
-  /* MUST copy the "Timestamp" header here (+ add a delay if necessary!)        */
-  /* a delay should not be necessary for 100 as it is sent in less than one sec */
-
-
-  return resp_100;
-icr_error:
-  osip_message_free (resp_100);
-  return NULL;
-}
+#endif
 
 static void
 ist_handle_transport_error (osip_transaction_t * ist, int err)
@@ -188,7 +224,6 @@ void
 ist_rcv_invite (osip_transaction_t * ist, osip_event_t * evt)
 {
   int i;
-  osip_t *osip = (osip_t *) ist->config;
 
   if (ist->state == IST_PRE_PROCEEDING) /* announce new INVITE */
     {
@@ -205,48 +240,7 @@ ist_rcv_invite (osip_transaction_t * ist, osip_event_t * evt)
                                ist->orig_request);
       if (ist->last_response != NULL)   /* retransmit last response */
         {
-          osip_via_t *via;
-
-          via = (osip_via_t *) osip_list_get (ist->last_response->vias, 0);
-          if (via)
-            {
-              char *host;
-              int port;
-              osip_generic_param_t *maddr;
-              osip_generic_param_t *received;
-              osip_generic_param_t *rport;
-
-              osip_via_param_get_byname (via, "maddr", &maddr);
-              osip_via_param_get_byname (via, "received", &received);
-              osip_via_param_get_byname (via, "rport", &rport);
-              /* 1: user should not use the provided information
-                 (host and port) if they are using a reliable
-                 transport. Instead, they should use the already
-                 open socket attached to this transaction. */
-              /* 2: check maddr and multicast usage */
-              if (maddr != NULL)
-                host = maddr->gvalue;
-              /* we should check if this is a multicast address and use
-                 set the "ttl" in this case. (this must be done in the
-                 UDP message (not at the SIP layer) */
-              else if (received != NULL)
-                host = received->gvalue;
-              else
-                host = via->host;
-
-              if (rport == NULL || rport->gvalue == NULL)
-                {
-                  if (via->port != NULL)
-                    port = osip_atoi (via->port);
-                  else
-                    port = 5060;
-              } else
-                port = osip_atoi (rport->gvalue);
-
-              i = osip->cb_send_message (ist, ist->last_response, host,
-                                         port, ist->out_socket);
-          } else
-            i = -1;
+	  i = __osip_transaction_snd_xxx(ist, ist->last_response);
           if (i != 0)
             {
               ist_handle_transport_error (ist, i);
@@ -274,8 +268,6 @@ ist_rcv_invite (osip_transaction_t * ist, osip_event_t * evt)
 void
 osip_ist_timeout_g_event (osip_transaction_t * ist, osip_event_t * evt)
 {
-  osip_via_t *via;
-  osip_t *osip = (osip_t *) ist->config;
   int i;
 
   ist->ist_context->timer_g_length = ist->ist_context->timer_g_length * 2;
@@ -285,47 +277,7 @@ osip_ist_timeout_g_event (osip_transaction_t * ist, osip_event_t * evt)
   add_gettimeofday (&ist->ist_context->timer_g_start,
                     ist->ist_context->timer_g_length);
 
-  /* retransmit RESPONSE */
-  via = (osip_via_t *) osip_list_get (ist->last_response->vias, 0);
-  if (via)
-    {
-      char *host;
-      int port;
-      osip_generic_param_t *maddr;
-      osip_generic_param_t *received;
-      osip_generic_param_t *rport;
-
-      osip_via_param_get_byname (via, "maddr", &maddr);
-      osip_via_param_get_byname (via, "received", &received);
-      osip_via_param_get_byname (via, "rport", &rport);
-      /* 1: user should not use the provided information
-         (host and port) if they are using a reliable
-         transport. Instead, they should use the already
-         open socket attached to this transaction. */
-      /* 2: check maddr and multicast usage */
-      if (maddr != NULL)
-        host = maddr->gvalue;
-      /* we should check if this is a multicast address and use
-         set the "ttl" in this case. (this must be done in the
-         UDP message (not at the SIP layer) */
-      else if (received != NULL)
-        host = received->gvalue;
-      else
-        host = via->host;
-
-      if (rport == NULL || rport->gvalue == NULL)
-        {
-          if (via->port != NULL)
-            port = osip_atoi (via->port);
-          else
-            port = 5060;
-      } else
-        port = osip_atoi (rport->gvalue);
-
-      i = osip->cb_send_message (ist, ist->last_response, host,
-                                 port, ist->out_socket);
-  } else
-    i = -1;
+  i = __osip_transaction_snd_xxx(ist, ist->last_response);
   if (i != 0)
     {
       ist_handle_transport_error (ist, i);
@@ -359,8 +311,6 @@ void
 ist_snd_1xx (osip_transaction_t * ist, osip_event_t * evt)
 {
   int i;
-  osip_via_t *via;
-  osip_t *osip = (osip_t *) ist->config;
 
   if (ist->last_response != NULL)
     {
@@ -368,46 +318,7 @@ ist_snd_1xx (osip_transaction_t * ist, osip_event_t * evt)
     }
   ist->last_response = evt->sip;
 
-  via = (osip_via_t *) osip_list_get (ist->last_response->vias, 0);
-  if (via)
-    {
-      char *host;
-      int port;
-      osip_generic_param_t *maddr;
-      osip_generic_param_t *received;
-      osip_generic_param_t *rport;
-
-      osip_via_param_get_byname (via, "maddr", &maddr);
-      osip_via_param_get_byname (via, "received", &received);
-      osip_via_param_get_byname (via, "rport", &rport);
-      /* 1: user should not use the provided information
-         (host and port) if they are using a reliable
-         transport. Instead, they should use the already
-         open socket attached to this transaction. */
-      /* 2: check maddr and multicast usage */
-      if (maddr != NULL)
-        host = maddr->gvalue;
-      /* we should check if this is a multicast address and use
-         set the "ttl" in this case. (this must be done in the
-         UDP message (not at the SIP layer) */
-      else if (received != NULL)
-        host = received->gvalue;
-      else
-        host = via->host;
-
-      if (rport == NULL || rport->gvalue == NULL)
-        {
-          if (via->port != NULL)
-            port = osip_atoi (via->port);
-          else
-            port = 5060;
-      } else
-        port = osip_atoi (rport->gvalue);
-
-      i = osip->cb_send_message (ist, ist->last_response, host,
-                                 port, ist->out_socket);
-  } else
-    i = -1;
+  i = __osip_transaction_snd_xxx(ist, evt->sip);
   if (i != 0)
     {
       ist_handle_transport_error (ist, i);
@@ -423,8 +334,6 @@ void
 ist_snd_2xx (osip_transaction_t * ist, osip_event_t * evt)
 {
   int i;
-  osip_via_t *via;
-  osip_t *osip = (osip_t *) ist->config;
 
   if (ist->last_response != NULL)
     {
@@ -432,45 +341,7 @@ ist_snd_2xx (osip_transaction_t * ist, osip_event_t * evt)
     }
   ist->last_response = evt->sip;
 
-  via = (osip_via_t *) osip_list_get (ist->last_response->vias, 0);
-  if (via)
-    {
-      char *host;
-      int port;
-      osip_generic_param_t *maddr;
-      osip_generic_param_t *received;
-      osip_generic_param_t *rport;
-
-      osip_via_param_get_byname (via, "maddr", &maddr);
-      osip_via_param_get_byname (via, "received", &received);
-      osip_via_param_get_byname (via, "rport", &rport);
-      /* 1: user should not use the provided information
-         (host and port) if they are using a reliable
-         transport. Instead, they should use the already
-         open socket attached to this transaction. */
-      /* 2: check maddr and multicast usage */
-      if (maddr != NULL)
-        host = maddr->gvalue;
-      /* we should check if this is a multicast address and use
-         set the "ttl" in this case. (this must be done in the
-         UDP message (not at the SIP layer) */
-      else if (received != NULL)
-        host = received->gvalue;
-      else
-        host = via->host;
-
-      if (rport == NULL || rport->gvalue == NULL)
-        {
-          if (via->port != NULL)
-            port = osip_atoi (via->port);
-          else
-            port = 5060;
-      } else
-        port = osip_atoi (rport->gvalue);
-      i = osip->cb_send_message (ist, ist->last_response, host,
-                                 port, ist->out_socket);
-  } else
-    i = -1;
+  i = __osip_transaction_snd_xxx(ist, evt->sip);
   if (i != 0)
     {
       ist_handle_transport_error (ist, i);
@@ -488,8 +359,6 @@ void
 ist_snd_3456xx (osip_transaction_t * ist, osip_event_t * evt)
 {
   int i;
-  osip_via_t *via;
-  osip_t *osip = (osip_t *) ist->config;
 
   if (ist->last_response != NULL)
     {
@@ -497,45 +366,7 @@ ist_snd_3456xx (osip_transaction_t * ist, osip_event_t * evt)
     }
   ist->last_response = evt->sip;
 
-  via = (osip_via_t *) osip_list_get (ist->last_response->vias, 0);
-  if (via)
-    {
-      char *host;
-      int port;
-      osip_generic_param_t *maddr;
-      osip_generic_param_t *received;
-      osip_generic_param_t *rport;
-
-      osip_via_param_get_byname (via, "maddr", &maddr);
-      osip_via_param_get_byname (via, "received", &received);
-      osip_via_param_get_byname (via, "rport", &rport);
-      /* 1: user should not use the provided information
-         (host and port) if they are using a reliable
-         transport. Instead, they should use the already
-         open socket attached to this transaction. */
-      /* 2: check maddr and multicast usage */
-      if (maddr != NULL)
-        host = maddr->gvalue;
-      /* we should check if this is a multicast address and use
-         set the "ttl" in this case. (this must be done in the
-         UDP message (not at the SIP layer) */
-      else if (received != NULL)
-        host = received->gvalue;
-      else
-        host = via->host;
-
-      if (rport == NULL || rport->gvalue == NULL)
-        {
-          if (via->port != NULL)
-            port = osip_atoi (via->port);
-          else
-            port = 5060;
-      } else
-        port = osip_atoi (rport->gvalue);
-      i = osip->cb_send_message (ist, ist->last_response, host,
-                                 port, ist->out_socket);
-  } else
-    i = -1;
+  i = __osip_transaction_snd_xxx(ist, evt->sip);
   if (i != 0)
     {
       ist_handle_transport_error (ist, i);
