@@ -1,6 +1,6 @@
 /*
   The oSIP library implements the Session Initiation Protocol (SIP -rfc3261-)
-  Copyright (C) 2001,2002,2003  Aymeric MOIZARD jack@atosc.org
+  Copyright (C) 2001,2002,2003,2004,2005,2006,2007 Aymeric MOIZARD jack@atosc.org
   
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -45,10 +45,12 @@ static int
 __osip_global_init ()
 {
   /* load the fsm configuration */
+#ifndef MINISIZE
   __ict_load_fsm ();
   __ist_load_fsm ();
   __nict_load_fsm ();
   __nist_load_fsm ();
+#endif
 
   /* load the parser configuration */
   parser_init ();
@@ -68,10 +70,12 @@ __osip_global_init ()
 static void
 __osip_global_free ()
 {
+#ifndef MINISIZE
   __ict_unload_fsm ();
   __ist_unload_fsm ();
   __nict_unload_fsm ();
   __nist_unload_fsm ();
+#endif
 
 #ifdef OSIP_MT
   osip_mutex_destroy (ict_fastmutex);
@@ -91,7 +95,7 @@ osip_response_get_destination (osip_message_t * response, char **address,
   char *host = NULL;
   int port = 0;
 
-  via = (osip_via_t *) osip_list_get (response->vias, 0);
+  via = (osip_via_t *) osip_list_get (&response->vias, 0);
   if (via)
     {
       osip_generic_param_t *maddr;
@@ -157,9 +161,9 @@ osip_ixt_unlock (osip_t * osip)
 void
 osip_add_ixt (osip_t * osip, ixt_t * ixt)
 {
-  /* ajout dans la liste de osip_t->ixt */
+  /* add in list osip_t->ixt */
   osip_ixt_lock (osip);
-  osip_list_add (osip->ixt_retransmissions, (void *) ixt, 0);
+  osip_list_add (&osip->ixt_retransmissions, (void *) ixt, 0);
   osip_ixt_unlock (osip);
 }
 
@@ -170,14 +174,14 @@ osip_remove_ixt (osip_t * osip, ixt_t * ixt)
   int found = 0;
   ixt_t *tmp;
 
-  /* ajout dans la liste de osip_t->ixt */
+  /* add in list osip_t->ixt */
   osip_ixt_lock (osip);
-  for (i = 0; !osip_list_eol (osip->ixt_retransmissions, i); i++)
+  for (i = 0; !osip_list_eol (&osip->ixt_retransmissions, i); i++)
     {
-      tmp = (ixt_t *) osip_list_get (osip->ixt_retransmissions, i);
+      tmp = (ixt_t *) osip_list_get (&osip->ixt_retransmissions, i);
       if (tmp == ixt)
         {
-          osip_list_remove (osip->ixt_retransmissions, i);
+          osip_list_remove (&osip->ixt_retransmissions, i);
           found = 1;
           break;
         }
@@ -196,9 +200,10 @@ ixt_init (ixt_t ** ixt)
   pixt->dialog = NULL;
   pixt->msg2xx = NULL;
   pixt->ack = NULL;
-  pixt->start = time (NULL);
-  pixt->interval = 500;
-  pixt->counter = 7;
+  pixt->interval = DEFAULT_T1;
+  osip_gettimeofday(&pixt->start, NULL);
+  add_gettimeofday(&pixt->start, pixt->interval+10);
+  pixt->counter = 10;
   pixt->dest = NULL;
   pixt->port = 5060;
   pixt->sock = -1;
@@ -257,14 +262,14 @@ osip_stop_200ok_retransmissions (osip_t * osip, osip_message_t * ack)
   ixt_t *ixt;
 
   osip_ixt_lock (osip);
-  for (i = 0; !osip_list_eol (osip->ixt_retransmissions, i); i++)
+  for (i = 0; !osip_list_eol (&osip->ixt_retransmissions, i); i++)
     {
-      ixt = (ixt_t *) osip_list_get (osip->ixt_retransmissions, i);
+      ixt = (ixt_t *) osip_list_get (&osip->ixt_retransmissions, i);
       if (osip_dialog_match_as_uas (ixt->dialog, ack) == 0)
         {
-          osip_list_remove (osip->ixt_retransmissions, i);
-          ixt_free (ixt);
+          osip_list_remove (&osip->ixt_retransmissions, i);
           dialog = ixt->dialog;
+          ixt_free (ixt);
           break;
         }
     }
@@ -281,12 +286,12 @@ osip_stop_retransmissions_from_dialog (osip_t * osip, osip_dialog_t * dialog)
   ixt_t *ixt;
 
   osip_ixt_lock (osip);
-  for (i = 0; !osip_list_eol (osip->ixt_retransmissions, i); i++)
+  for (i = 0; !osip_list_eol (&osip->ixt_retransmissions, i); i++)
     {
-      ixt = (ixt_t *) osip_list_get (osip->ixt_retransmissions, i);
+      ixt = (ixt_t *) osip_list_get (&osip->ixt_retransmissions, i);
       if (ixt->dialog == dialog)
         {
-          osip_list_remove (osip->ixt_retransmissions, i);
+          osip_list_remove (&osip->ixt_retransmissions, i);
           ixt_free (ixt);
           i--;
         }
@@ -295,12 +300,14 @@ osip_stop_retransmissions_from_dialog (osip_t * osip, osip_dialog_t * dialog)
 }
 
 void
-ixt_retransmit (osip_t * osip, ixt_t * ixt, time_t current)
+ixt_retransmit (osip_t * osip, ixt_t * ixt, struct timeval *current)
 {
-  if ((current - ixt->start) * 1000 > ixt->interval)
+  if (osip_timercmp(current, &ixt->start, >))
     {
       ixt->interval = ixt->interval * 2;
-      ixt->start = current;
+      if (ixt->interval > 4000)
+         ixt->interval = 4000;
+      add_gettimeofday (&ixt->start, ixt->interval);
       if (ixt->ack != NULL)
         osip->cb_send_message (NULL, ixt->ack, ixt->dest, ixt->port, ixt->sock);
       else if (ixt->msg2xx != NULL)
@@ -313,19 +320,20 @@ void
 osip_retransmissions_execute (osip_t * osip)
 {
   int i;
-  time_t current;
   ixt_t *ixt;
+  struct timeval current;
+
+  osip_gettimeofday (&current, NULL);
 
-  current = time (NULL);
   osip_ixt_lock (osip);
-  for (i = 0; !osip_list_eol (osip->ixt_retransmissions, i); i++)
+  for (i = 0; !osip_list_eol (&osip->ixt_retransmissions, i); i++)
     {
-      ixt = (ixt_t *) osip_list_get (osip->ixt_retransmissions, i);
-      ixt_retransmit (osip, ixt, current);
+      ixt = (ixt_t *) osip_list_get (&osip->ixt_retransmissions, i);
+      ixt_retransmit (osip, ixt, &current);
       if (ixt->counter == 0)
         {
           /* remove it */
-          osip_list_remove (osip->ixt_retransmissions, i);
+          osip_list_remove (&osip->ixt_retransmissions, i);
           ixt_free (ixt);
           i--;
         }
@@ -467,7 +475,7 @@ __osip_add_ict (osip_t * osip, osip_transaction_t * ict)
       }
   }
 #endif
-  osip_list_add (osip->osip_ict_transactions, ict, -1);
+  osip_list_add (&osip->osip_ict_transactions, ict, -1);
 #ifdef OSIP_MT
   osip_mutex_unlock (ict_fastmutex);
 #endif
@@ -508,7 +516,7 @@ __osip_add_ist (osip_t * osip, osip_transaction_t * ist)
       }
   }
 #endif
-  osip_list_add (osip->osip_ist_transactions, ist, -1);
+  osip_list_add (&osip->osip_ist_transactions, ist, -1);
 #ifdef OSIP_MT
   osip_mutex_unlock (ist_fastmutex);
 #endif
@@ -550,7 +558,7 @@ __osip_add_nict (osip_t * osip, osip_transaction_t * nict)
       }
   }
 #endif
-  osip_list_add (osip->osip_nict_transactions, nict, -1);
+  osip_list_add (&osip->osip_nict_transactions, nict, -1);
 #ifdef OSIP_MT
   osip_mutex_unlock (nict_fastmutex);
 #endif
@@ -591,7 +599,7 @@ __osip_add_nist (osip_t * osip, osip_transaction_t * nist)
       }
   }
 #endif
-  osip_list_add (osip->osip_nist_transactions, nist, -1);
+  osip_list_add (&osip->osip_nist_transactions, nist, -1);
 #ifdef OSIP_MT
   osip_mutex_unlock (nist_fastmutex);
 #endif
@@ -654,7 +662,7 @@ __osip_remove_ict_transaction (osip_t * osip, osip_transaction_t * ict)
 #endif
 
   tmp =
-    (osip_transaction_t *) osip_list_get_first (osip->osip_ict_transactions,
+    (osip_transaction_t *) osip_list_get_first (&osip->osip_ict_transactions,
                                                 &iterator);
   while (osip_list_iterator_has_elem (iterator))
     {
@@ -711,7 +719,7 @@ __osip_remove_ist_transaction (osip_t * osip, osip_transaction_t * ist)
 #endif
 
   tmp =
-    (osip_transaction_t *) osip_list_get_first (osip->osip_ist_transactions,
+    (osip_transaction_t *) osip_list_get_first (&osip->osip_ist_transactions,
                                                 &iterator);
   while (osip_list_iterator_has_elem (iterator))
     {
@@ -768,7 +776,7 @@ __osip_remove_nict_transaction (osip_t * osip, osip_transaction_t * nict)
 #endif
 
   tmp =
-    (osip_transaction_t *) osip_list_get_first (osip->osip_nict_transactions,
+    (osip_transaction_t *) osip_list_get_first (&osip->osip_nict_transactions,
                                                 &iterator);
   while (osip_list_iterator_has_elem (iterator))
     {
@@ -825,7 +833,7 @@ __osip_remove_nist_transaction (osip_t * osip, osip_transaction_t * nist)
 #endif
 
   tmp =
-    (osip_transaction_t *) osip_list_get_first (osip->osip_nist_transactions,
+    (osip_transaction_t *) osip_list_get_first (&osip->osip_nist_transactions,
                                                 &iterator);
   while (osip_list_iterator_has_elem (iterator))
     {
@@ -1022,13 +1030,13 @@ __osip_find_transaction (osip_t * osip, osip_event_t * evt, int consume)
           if (0 == strcmp (evt->sip->cseq->method, "INVITE")
               || 0 == strcmp (evt->sip->cseq->method, "ACK"))
             {
-              transactions = osip->osip_ist_transactions;
+              transactions = &osip->osip_ist_transactions;
 #ifdef OSIP_MT
               mut = ist_fastmutex;
 #endif
           } else
             {
-              transactions = osip->osip_nist_transactions;
+              transactions = &osip->osip_nist_transactions;
 #ifdef OSIP_MT
               mut = nist_fastmutex;
 #endif
@@ -1037,13 +1045,13 @@ __osip_find_transaction (osip_t * osip, osip_event_t * evt, int consume)
         {
           if (0 == strcmp (evt->sip->cseq->method, "INVITE"))
             {
-              transactions = osip->osip_ict_transactions;
+              transactions = &osip->osip_ict_transactions;
 #ifdef OSIP_MT
               mut = ict_fastmutex;
 #endif
           } else
             {
-              transactions = osip->osip_nict_transactions;
+              transactions = &osip->osip_nict_transactions;
 #ifdef OSIP_MT
               mut = nict_fastmutex;
 #endif
@@ -1055,13 +1063,13 @@ __osip_find_transaction (osip_t * osip, osip_event_t * evt, int consume)
         {
           if (0 == strcmp (evt->sip->cseq->method, "INVITE"))
             {
-              transactions = osip->osip_ist_transactions;
+              transactions = &osip->osip_ist_transactions;
 #ifdef OSIP_MT
               mut = ist_fastmutex;
 #endif
           } else
             {
-              transactions = osip->osip_nist_transactions;
+              transactions = &osip->osip_nist_transactions;
 #ifdef OSIP_MT
               mut = nist_fastmutex;
 #endif
@@ -1071,13 +1079,13 @@ __osip_find_transaction (osip_t * osip, osip_event_t * evt, int consume)
           if (0 == strcmp (evt->sip->cseq->method, "INVITE")
               || 0 == strcmp (evt->sip->cseq->method, "ACK"))
             {
-              transactions = osip->osip_ict_transactions;
+              transactions = &osip->osip_ict_transactions;
 #ifdef OSIP_MT
               mut = ict_fastmutex;
 #endif
           } else
             {
-              transactions = osip->osip_nict_transactions;
+              transactions = &osip->osip_nict_transactions;
 #ifdef OSIP_MT
               mut = nict_fastmutex;
 #endif
@@ -1364,22 +1372,12 @@ osip_init (osip_t ** osip)
 
   memset (*osip, 0, sizeof (osip_t));
 
-  (*osip)->osip_ict_transactions =
-    (osip_list_t *) osip_malloc (sizeof (osip_list_t));
-  osip_list_init ((*osip)->osip_ict_transactions);
-  (*osip)->osip_ist_transactions =
-    (osip_list_t *) osip_malloc (sizeof (osip_list_t));
-  osip_list_init ((*osip)->osip_ist_transactions);
-  (*osip)->osip_nict_transactions =
-    (osip_list_t *) osip_malloc (sizeof (osip_list_t));
-  osip_list_init ((*osip)->osip_nict_transactions);
-  (*osip)->osip_nist_transactions =
-    (osip_list_t *) osip_malloc (sizeof (osip_list_t));
-  osip_list_init ((*osip)->osip_nist_transactions);
+  osip_list_init (&(*osip)->osip_ict_transactions);
+  osip_list_init (&(*osip)->osip_ist_transactions);
+  osip_list_init (&(*osip)->osip_nict_transactions);
+  osip_list_init (&(*osip)->osip_nist_transactions);
 
-  (*osip)->ixt_retransmissions =
-    (osip_list_t *) osip_malloc (sizeof (osip_list_t));
-  osip_list_init ((*osip)->ixt_retransmissions);
+  osip_list_init (&(*osip)->ixt_retransmissions);
 
 #if defined(HAVE_DICT_DICT_H)
   (*osip)->osip_ict_hastable = hashtable_dict_new ((dict_cmp_func) strcmp,
@@ -1407,13 +1405,6 @@ osip_init (osip_t ** osip)
 void
 osip_release (osip_t * osip)
 {
-  osip_free (osip->osip_ict_transactions);
-  osip_free (osip->osip_ist_transactions);
-  osip_free (osip->osip_nict_transactions);
-  osip_free (osip->osip_nist_transactions);
-
-  osip_free (osip->ixt_retransmissions);
-
   osip_free (osip);
   decrease_ref_count ();
 }
@@ -1448,7 +1439,7 @@ osip_ict_execute (osip_t * osip)
 #ifdef OSIP_MT
   osip_mutex_lock (ict_fastmutex);
 #endif
-  len = osip_list_size (osip->osip_ict_transactions);
+  len = osip_list_size (&osip->osip_ict_transactions);
   if (0 >= len)
     {
 #ifdef OSIP_MT
@@ -1457,8 +1448,15 @@ osip_ict_execute (osip_t * osip)
       return 0;
     }
   array = osip_malloc (sizeof (void *) * len);
+  if (array==NULL)
+    {
+#ifdef OSIP_MT
+      osip_mutex_unlock (ict_fastmutex);
+#endif
+      return 0;
+    }
   transaction =
-    (osip_transaction_t *) osip_list_get_first (osip->osip_ict_transactions,
+    (osip_transaction_t *) osip_list_get_first (&osip->osip_ict_transactions,
                                                 &iterator);
   while (osip_list_iterator_has_elem (iterator))
     {
@@ -1504,7 +1502,7 @@ osip_ist_execute (osip_t * osip)
 #ifdef OSIP_MT
   osip_mutex_lock (ist_fastmutex);
 #endif
-  len = osip_list_size (osip->osip_ist_transactions);
+  len = osip_list_size (&osip->osip_ist_transactions);
   if (0 >= len)
     {
 #ifdef OSIP_MT
@@ -1513,8 +1511,15 @@ osip_ist_execute (osip_t * osip)
       return 0;
     }
   array = osip_malloc (sizeof (void *) * len);
+  if (array==NULL)
+    {
+#ifdef OSIP_MT
+      osip_mutex_unlock (ist_fastmutex);
+#endif
+      return 0;
+    }
   transaction =
-    (osip_transaction_t *) osip_list_get_first (osip->osip_ist_transactions,
+    (osip_transaction_t *) osip_list_get_first (&osip->osip_ist_transactions,
                                                 &iterator);
   while (osip_list_iterator_has_elem (iterator))
     {
@@ -1561,7 +1566,7 @@ osip_nict_execute (osip_t * osip)
 #ifdef OSIP_MT
   osip_mutex_lock (nict_fastmutex);
 #endif
-  len = osip_list_size (osip->osip_nict_transactions);
+  len = osip_list_size (&osip->osip_nict_transactions);
   if (0 >= len)
     {
 #ifdef OSIP_MT
@@ -1570,8 +1575,15 @@ osip_nict_execute (osip_t * osip)
       return 0;
     }
   array = osip_malloc (sizeof (void *) * len);
+  if (array==NULL)
+    {
+#ifdef OSIP_MT
+      osip_mutex_unlock (nict_fastmutex);
+#endif
+      return 0;
+    }
   transaction =
-    (osip_transaction_t *) osip_list_get_first (osip->osip_nict_transactions,
+    (osip_transaction_t *) osip_list_get_first (&osip->osip_nict_transactions,
                                                 &iterator);
   while (osip_list_iterator_has_elem (iterator))
     {
@@ -1617,7 +1629,7 @@ osip_nist_execute (osip_t * osip)
 #ifdef OSIP_MT
   osip_mutex_lock (nist_fastmutex);
 #endif
-  len = osip_list_size (osip->osip_nist_transactions);
+  len = osip_list_size (&osip->osip_nist_transactions);
   if (0 >= len)
     {
 #ifdef OSIP_MT
@@ -1626,8 +1638,15 @@ osip_nist_execute (osip_t * osip)
       return 0;
     }
   array = osip_malloc (sizeof (void *) * len);
+  if (array==NULL)
+    {
+#ifdef OSIP_MT
+      osip_mutex_unlock (nist_fastmutex);
+#endif
+      return 0;
+    }
   transaction =
-    (osip_transaction_t *) osip_list_get_first (osip->osip_nist_transactions,
+    (osip_transaction_t *) osip_list_get_first (&osip->osip_nist_transactions,
                                                 &iterator);
   while (osip_list_iterator_has_elem (iterator))
     {
@@ -1674,7 +1693,7 @@ osip_timers_gettimeout (osip_t * osip, struct timeval *lower_tv)
 #endif
   /* handle ict timers */
   tr =
-    (osip_transaction_t *) osip_list_get_first (osip->osip_ict_transactions,
+    (osip_transaction_t *) osip_list_get_first (&osip->osip_ict_transactions,
                                                 &iterator);
   while (osip_list_iterator_has_elem (iterator))
     {
@@ -1718,7 +1737,7 @@ osip_timers_gettimeout (osip_t * osip, struct timeval *lower_tv)
 #endif
   /* handle ist timers */
   tr =
-    (osip_transaction_t *) osip_list_get_first (osip->osip_ist_transactions,
+    (osip_transaction_t *) osip_list_get_first (&osip->osip_ist_transactions,
                                                 &iterator);
   while (osip_list_iterator_has_elem (iterator))
     {
@@ -1748,7 +1767,7 @@ osip_timers_gettimeout (osip_t * osip, struct timeval *lower_tv)
 #endif
   /* handle nict timers */
   tr =
-    (osip_transaction_t *) osip_list_get_first (osip->osip_nict_transactions,
+    (osip_transaction_t *) osip_list_get_first (&osip->osip_nict_transactions,
                                                 &iterator);
   while (osip_list_iterator_has_elem (iterator))
     {
@@ -1778,7 +1797,7 @@ osip_timers_gettimeout (osip_t * osip, struct timeval *lower_tv)
 #endif
   /* handle nist timers */
   tr =
-    (osip_transaction_t *) osip_list_get_first (osip->osip_nist_transactions,
+    (osip_transaction_t *) osip_list_get_first (&osip->osip_nist_transactions,
                                                 &iterator);
   while (osip_list_iterator_has_elem (iterator))
     {
@@ -1805,15 +1824,19 @@ osip_timers_gettimeout (osip_t * osip, struct timeval *lower_tv)
   {
     ixt_t *ixt;
 
-    ixt = (ixt_t *) osip_list_get_first (osip->ixt_retransmissions, &iterator);
+    ixt = (ixt_t *) osip_list_get_first (&osip->ixt_retransmissions, &iterator);
     while (osip_list_iterator_has_elem (iterator))
       {
-        struct timeval cmpTime;
-        div_t dValue = div (ixt->interval, 1000);
-
-        cmpTime.tv_sec = (long) (ixt->start + dValue.quot);
-        cmpTime.tv_usec = dValue.rem * 1000;
-        min_timercmp (lower_tv, &cmpTime);
+        min_timercmp (lower_tv, &ixt->start);
+	if (osip_timercmp (&now, lower_tv, >))
+	  {
+	    lower_tv->tv_sec = 0;
+	    lower_tv->tv_usec = 0;
+#ifdef OSIP_MT
+	    osip_mutex_unlock (ixt_fastmutex);
+#endif
+	    return;
+	  }
 
         ixt = (ixt_t *) osip_list_get_next (&iterator);
       }
@@ -1855,7 +1878,7 @@ osip_timers_ict_execute (osip_t * osip)
 #endif
   /* handle ict timers */
   tr =
-    (osip_transaction_t *) osip_list_get_first (osip->osip_ict_transactions,
+    (osip_transaction_t *) osip_list_get_first (&osip->osip_ict_transactions,
                                                 &iterator);
   while (osip_list_iterator_has_elem (iterator))
     {
@@ -1906,7 +1929,7 @@ osip_timers_ist_execute (osip_t * osip)
 #endif
   /* handle ist timers */
   tr =
-    (osip_transaction_t *) osip_list_get_first (osip->osip_ist_transactions,
+    (osip_transaction_t *) osip_list_get_first (&osip->osip_ist_transactions,
                                                 &iterator);
   while (osip_list_iterator_has_elem (iterator))
     {
@@ -1948,7 +1971,7 @@ osip_timers_nict_execute (osip_t * osip)
 #endif
   /* handle nict timers */
   tr =
-    (osip_transaction_t *) osip_list_get_first (osip->osip_nict_transactions,
+    (osip_transaction_t *) osip_list_get_first (&osip->osip_nict_transactions,
                                                 &iterator);
   while (osip_list_iterator_has_elem (iterator))
     {
@@ -1992,7 +2015,7 @@ osip_timers_nist_execute (osip_t * osip)
 #endif
   /* handle nist timers */
   tr =
-    (osip_transaction_t *) osip_list_get_first (osip->osip_nist_transactions,
+    (osip_transaction_t *) osip_list_get_first (&osip->osip_nist_transactions,
                                                 &iterator);
   while (osip_list_iterator_has_elem (iterator))
     {
@@ -2046,6 +2069,7 @@ __osip_kill_transaction_callback (int type, osip_transaction_t * tr)
                    "invalid callback type %d\n", type));
       return;
     }
+  tr->completed_time = time (NULL);
   if (config->kill_callbacks[type] == NULL)
     return;
   config->kill_callbacks[type] (type, tr);
