@@ -1,6 +1,6 @@
 /*
   The oSIP library implements the Session Initiation Protocol (SIP -rfc3261-)
-  Copyright (C) 2001,2002,2003,2004,2005  Aymeric MOIZARD jack@atosc.org
+  Copyright (C) 2001,2002,2003,2004,2005,2006,2007 Aymeric MOIZARD jack@atosc.org
   
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -24,7 +24,7 @@
 #include <osipparser2/osip_message.h>
 #include <osipparser2/osip_parser.h>
 
-
+#ifndef MINISIZE
 
 /* adds the accept header to message.              */
 /* INPUT : char *hvalue | value of header.    */
@@ -36,11 +36,6 @@ osip_message_set_accept (osip_message_t * sip, const char *hvalue)
   osip_accept_t *accept;
   int i;
 
-#if 0
-  if (hvalue == NULL || hvalue[0] == '\0')
-    return 0;
-#endif
-
   i = accept_init (&accept);
   if (i != 0)
     return -1;
@@ -52,7 +47,7 @@ osip_message_set_accept (osip_message_t * sip, const char *hvalue)
     }
   sip->message_property = 2;
 
-  osip_list_add (sip->accepts, accept, -1);
+  osip_list_add (&sip->accepts, accept, -1);
   return 0;
 }
 
@@ -64,9 +59,9 @@ osip_message_get_accept (const osip_message_t * sip, int pos,
   osip_accept_t *accept;
 
   *dest = NULL;
-  if (osip_list_size (sip->accepts) <= pos)
+  if (osip_list_size (&sip->accepts) <= pos)
     return -1;                  /* does not exist */
-  accept = (osip_accept_t *) osip_list_get (sip->accepts, pos);
+  accept = (osip_accept_t *) osip_list_get (&sip->accepts, pos);
   *dest = accept;
   return pos;
 }
@@ -79,13 +74,19 @@ osip_accept_to_str (const osip_accept_t * accept, char **dest)
 {
   char *buf;
   char *tmp;
-  size_t len;
+  size_t len = 0;
 
   *dest = NULL;
   if (accept == NULL)
     return -1;
 
-  if ((accept->type == NULL) && (accept->subtype == NULL))
+  if (accept->type != NULL)
+    len += strlen (accept->type);
+
+  if (accept->subtype != NULL)
+    len += strlen (accept->subtype);
+
+  if (len == 0)
     {
       /* Empty header ! */
       buf = (char *) osip_malloc (2);
@@ -96,8 +97,8 @@ osip_accept_to_str (const osip_accept_t * accept, char **dest)
     }
 
   /* try to guess a long enough length */
-  len = strlen (accept->type) + strlen (accept->subtype) + 4    /* for '/', ' ', ';' and '\0' */
-    + 10 * osip_list_size (accept->gen_params);
+  len += 4    /* for '/', ' ', ';' and '\0' */
+    + 10 * osip_list_size (&accept->gen_params);
 
   buf = (char *) osip_malloc (len);
   tmp = buf;
@@ -110,17 +111,17 @@ osip_accept_to_str (const osip_accept_t * accept, char **dest)
     osip_generic_param_t *u_param;
 
 #if 0
-    if (!osip_list_eol (accept->gen_params, pos))
+    if (!osip_list_eol (&accept->gen_params, pos))
       {                         /* needed for cannonical form! (authentication issue of rfc2543) */
         sprintf (tmp, " ");
         tmp++;
       }
 #endif
-    while (!osip_list_eol (accept->gen_params, pos))
+    while (!osip_list_eol (&accept->gen_params, pos))
       {
         size_t tmp_len;
 
-        u_param = (osip_generic_param_t *) osip_list_get (accept->gen_params, pos);
+        u_param = (osip_generic_param_t *) osip_list_get (&accept->gen_params, pos);
         if (u_param->gvalue == NULL)
           {
             osip_free (buf);
@@ -142,3 +143,5 @@ osip_accept_to_str (const osip_accept_t * accept, char **dest)
   *dest = buf;
   return 0;
 }
+
+#endif
