@@ -1,6 +1,6 @@
 /*
   The oSIP library implements the Session Initiation Protocol (SIP -rfc3261-)
-  Copyright (C) 2001,2002,2003,2004,2005  Aymeric MOIZARD jack@atosc.org
+  Copyright (C) 2001,2002,2003,2004,2005,2006,2007 Aymeric MOIZARD jack@atosc.org
   
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -240,6 +240,7 @@ osip_authorization_parse (osip_authorization_t * auth, const char *hvalue)
   return 0;                     /* ok */
 }
 
+#ifndef MINISIZE
 /* returns the authorization header.   */
 /* INPUT : osip_message_t *sip | sip message.   */
 /* returns null on error. */
@@ -257,6 +258,7 @@ osip_message_get_authorization (const osip_message_t * sip, int pos,
   *dest = authorization;
   return pos;
 }
+#endif
 
 char *
 osip_authorization_get_auth_type (const osip_authorization_t * authorization)
