@@ -1,6 +1,6 @@
 /*
   The oSIP library implements the Session Initiation Protocol (SIP -rfc3261-)
-  Copyright (C) 2001,2002,2003,2004,2005  Aymeric MOIZARD jack@atosc.org
+  Copyright (C) 2001,2002,2003,2004,2005,2006,2007 Aymeric MOIZARD jack@atosc.org
   
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -52,6 +52,7 @@ osip_message_set_call_id (osip_message_t * sip, const char *hvalue)
   return 0;
 }
 
+#ifndef MINISIZE
 /* returns the call_id.               */
 /* INPUT : osip_message_t *sip | sip message.  */
 osip_call_id_t *
@@ -59,6 +60,7 @@ osip_message_get_call_id (const osip_message_t * sip)
 {
   return sip->call_id;
 }
+#endif
 
 int
 osip_call_id_init (osip_call_id_t ** callid)
