@@ -1,6 +1,6 @@
 /*
   The oSIP library implements the Session Initiation Protocol (SIP -rfc3261-)
-  Copyright (C) 2001,2002,2003,2004,2005  Aymeric MOIZARD jack@atosc.org
+  Copyright (C) 2001,2002,2003,2004,2005,2006,2007 Aymeric MOIZARD jack@atosc.org
   
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -25,6 +25,8 @@
 #include <osipparser2/osip_parser.h>
 #include "parser.h"
 
+#ifndef MINISIZE
+
 /* content-Encoding = token
    token possible values are gzip,compress,deflate
 */
@@ -47,7 +49,7 @@ osip_message_set_content_encoding (osip_message_t * sip, const char *hvalue)
       return -1;
     }
   sip->message_property = 2;
-  osip_list_add (sip->content_encodings, content_encoding, -1);
+  osip_list_add (&sip->content_encodings, content_encoding, -1);
   return 0;
 }
 
@@ -58,9 +60,11 @@ osip_message_get_content_encoding (const osip_message_t * sip, int pos,
   osip_content_encoding_t *ce;
 
   *dest = NULL;
-  if (osip_list_size (sip->content_encodings) <= pos)
+  if (osip_list_size (&sip->content_encodings) <= pos)
     return -1;                  /* does not exist */
-  ce = (osip_content_encoding_t *) osip_list_get (sip->content_encodings, pos);
+  ce = (osip_content_encoding_t *) osip_list_get (&sip->content_encodings, pos);
   *dest = ce;
   return pos;
 }
+
+#endif
