@@ -1,6 +1,6 @@
 /*
   The oSIP library implements the Session Initiation Protocol (SIP -rfc3261-)
-  Copyright (C) 2001,2002,2003,2004,2005  Aymeric MOIZARD jack@atosc.org
+  Copyright (C) 2001,2002,2003,2004,2005,2006,2007 Aymeric MOIZARD jack@atosc.org
   
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -63,6 +63,7 @@ osip_message_set_from (osip_message_t * sip, const char *hvalue)
 }
 
 
+#ifndef MINISIZE
 /* returns the from header.            */
 /* INPUT : osip_message_t *sip | sip message.   */
 /* returns null on error. */
@@ -71,6 +72,7 @@ osip_message_get_from (const osip_message_t * sip)
 {
   return sip->from;
 }
+#endif
 
 int
 osip_from_init (osip_from_t ** from)
@@ -81,13 +83,7 @@ osip_from_init (osip_from_t ** from)
   (*from)->displayname = NULL;
   (*from)->url = NULL;
 
-  (*from)->gen_params = (osip_list_t *) osip_malloc (sizeof (osip_list_t));
-  if ((*from)->gen_params == NULL)
-    {
-      osip_free (*from);
-      *from = NULL;
-    }
-  osip_list_init ((*from)->gen_params);
+  osip_list_init (&(*from)->gen_params);
 
   return 0;
 }
@@ -105,7 +101,7 @@ osip_from_free (osip_from_t * from)
     }
   osip_free (from->displayname);
 
-  osip_generic_param_freelist (from->gen_params);
+  osip_generic_param_freelist (&from->gen_params);
 
   osip_free (from);
 }
@@ -178,13 +174,15 @@ osip_from_parse (osip_from_t * from, const char *hvalue)
       if ((displayname != NULL) && (url != NULL))
         {                       /* displayname IS A quoted-string (not a '*token') */
           const char *first;
-          const char *second;
+          const char *second=NULL;
 
           /* search for quotes */
           first = __osip_quote_find (hvalue);
-          second = __osip_quote_find (first + 1);
+          if (first == NULL)
+            return -1;          /* missing quote */
+	  second = __osip_quote_find (first + 1);
           if (second == NULL)
-            return -1;          /* if there is only 1 quote: failure */
+            return -1;          /* missing quote */
           if ((first > url))
             return -1;
 
@@ -230,7 +228,7 @@ osip_from_parse (osip_from_t * from, const char *hvalue)
     }
 
   if (gen_params != NULL)       /* now we are sure a param exist */
-    if (__osip_generic_param_parseall (from->gen_params, gen_params) == -1)
+    if (__osip_generic_param_parseall (&from->gen_params, gen_params) == -1)
       {
         return -1;
       }
@@ -304,9 +302,9 @@ osip_from_to_str (const osip_from_t * from, char **dest)
     size_t plen;
     char *tmp;
 
-    while (!osip_list_eol (from->gen_params, pos))
+    while (!osip_list_eol (&from->gen_params, pos))
       {
-        u_param = (osip_generic_param_t *) osip_list_get (from->gen_params, pos);
+        u_param = (osip_generic_param_t *) osip_list_get (&from->gen_params, pos);
 
         if (u_param->gvalue == NULL)
           plen = strlen (u_param->gname) + 2;
@@ -361,9 +359,9 @@ osip_from_param_get (osip_from_t * from, int pos, osip_generic_param_t ** fparam
   *fparam = NULL;
   if (from == NULL)
     return -1;
-  if (osip_list_size (from->gen_params) <= pos)
+  if (osip_list_size (&from->gen_params) <= pos)
     return -1;                  /* does not exist */
-  *fparam = (osip_generic_param_t *) osip_list_get (from->gen_params, pos);
+  *fparam = (osip_generic_param_t *) osip_list_get (&from->gen_params, pos);
   return pos;
 }
 
@@ -393,24 +391,12 @@ osip_from_clone (const osip_from_t * from, osip_from_t ** dest)
         }
     }
 
-  {
-    int pos = 0;
-    osip_generic_param_t *u_param;
-    osip_generic_param_t *dest_param;
-
-    while (!osip_list_eol (from->gen_params, pos))
-      {
-        u_param = (osip_generic_param_t *) osip_list_get (from->gen_params, pos);
-        i = osip_generic_param_clone (u_param, &dest_param);
-        if (i != 0)
-          {
-            osip_from_free (fr);
-            return -1;
-          }
-        osip_list_add (fr->gen_params, dest_param, -1);
-        pos++;
-      }
-  }
+  i = osip_list_clone(&from->gen_params, &fr->gen_params, (int *(*)(void *, void *)) &osip_generic_param_clone);
+  if (i != 0)
+    {
+      osip_from_free (fr);
+      return -1;
+    }
   *dest = fr;
   return 0;
 }
@@ -451,9 +437,9 @@ osip_from_compare (osip_from_t * from1, osip_from_t * from2)
     int pos = 0;
     osip_generic_param_t *u_param;
 
-    while (!osip_list_eol (from1->gen_params, pos))
+    while (!osip_list_eol (&from1->gen_params, pos))
       {
-        u_param = (osip_generic_param_t *) osip_list_get (from1->gen_params, pos);
+        u_param = (osip_generic_param_t *) osip_list_get (&from1->gen_params, pos);
         if (0 == strncmp (u_param->gname, "tag", 3))
           {
             tag1 = u_param->gvalue;
@@ -466,9 +452,9 @@ osip_from_compare (osip_from_t * from1, osip_from_t * from2)
     int pos = 0;
     osip_generic_param_t *u_param;
 
-    while (!osip_list_eol (from2->gen_params, pos))
+    while (!osip_list_eol (&from2->gen_params, pos))
       {
-        u_param = (osip_generic_param_t *) osip_list_get (from2->gen_params, pos);
+        u_param = (osip_generic_param_t *) osip_list_get (&from2->gen_params, pos);
         if (0 == strncmp (u_param->gname, "tag", 3))
           {
             tag2 = u_param->gvalue;
@@ -594,7 +580,10 @@ __osip_generic_param_parseall (osip_list_t * gen_params, const char *params)
     }
   pname = (char *) osip_malloc (equal - params);
   if (pname == NULL)
-    return -1;
+    {
+      osip_free (pvalue);
+      return -1;
+    }
   osip_strncpy (pname, params + 1, equal - params - 1);
 
   osip_generic_param_add (gen_params, pname, pvalue);
