@@ -1,6 +1,6 @@
 /*
   The oSIP library implements the Session Initiation Protocol (SIP -rfc3261-)
-  Copyright (C) 2001,2002,2003,2004,2005  Aymeric MOIZARD jack@atosc.org
+  Copyright (C) 2001,2002,2003,2004,2005,2006,2007 Aymeric MOIZARD jack@atosc.org
   
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -402,21 +402,16 @@ osip_message_set__header (osip_message_t * sip, const char *hname,
 
       ret = __osip_message_call_method (my_index, sip, hvalue);
       if (ret == -1)
-        {
-          OSIP_TRACE (osip_trace
-                      (__FILE__, __LINE__, OSIP_ERROR, NULL,
-                       "Could not set header: \"%s\" %s\n", hname, hvalue));
-          return -1;
-        }
+	return -1;
       return 0;
     }
   /* unknownheader */
   if (osip_message_set_header (sip, hname, hvalue) == -1)
     {
       OSIP_TRACE (osip_trace
-                  (__FILE__, __LINE__, OSIP_ERROR, NULL,
+                  (__FILE__, __LINE__, OSIP_WARNING, NULL,
                    "Could not set unknown header\n"));
-      return -1;
+      return 0;
     }
 
   return 0;
