@@ -1,6 +1,6 @@
 /*
   The oSIP library implements the Session Initiation Protocol (SIP -rfc3261-)
-  Copyright (C) 2001,2002,2003,2004,2005  Aymeric MOIZARD jack@atosc.org
+  Copyright (C) 2001,2002,2003,2004,2005,2006,2007 Aymeric MOIZARD jack@atosc.org
   
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -38,6 +38,7 @@ static int strcat_headers_one_per_line (char **_string, size_t * malloc_size,
                                         size_t size_of_header,
                                         int (*xxx_to_str) (void *, char **),
                                         char **next);
+#if 0
 static int strcat_headers_all_on_one_line (char **_string,
                                            size_t * malloc_size,
                                            char **_message,
@@ -47,6 +48,7 @@ static int strcat_headers_all_on_one_line (char **_string,
                                            int (*xxx_to_str) (void *,
                                                               char **),
                                            char **next);
+#endif
 
 static int
 __osip_message_startline_to_strreq (osip_message_t * sip, char **dest)
@@ -304,6 +306,7 @@ strcat_headers_one_per_line (char **_string, size_t * malloc_size,
   return 0;
 }
 
+#if 0
 static int
 strcat_headers_all_on_one_line (char **_string, size_t * malloc_size,
                                 char **_message, osip_list_t * headers,
@@ -383,6 +386,7 @@ strcat_headers_all_on_one_line (char **_string, size_t * malloc_size,
   *next = message;
   return 0;
 }
+#endif
 
  /* return values:
     1: structure and buffer "message" are identical.
@@ -492,164 +496,135 @@ _osip_message_to_str (osip_message_t * sip, char **dest,
       message = osip_strn_append (message, CRLF, 2);
     }
 
-  i =
-    strcat_headers_one_per_line (dest, &malloc_size, &message, sip->vias,
-                                 "Via: ", 5,
-                                 ((int (*)(void *, char **))
-                                  &osip_via_to_str), &next);
-  if (i != 0)
-    {
-      osip_free (*dest);
-      *dest = NULL;
-      return -1;
+  {
+    struct to_str_table {
+      char header_name[30];
+      int  header_length;
+      osip_list_t *header_list;
+      void *header_data;
+      int (*to_str) (void *, char **);
     }
-  message = next;
+#ifndef MINISIZE
+    table[25] = 
+#else
+    table[15] = 
+#endif
+	{
+	  { "Via: ", 5, NULL, NULL,
+	    (int (*)(void *, char **)) &osip_via_to_str },
+	  { "Record-Route: ", 14, NULL, NULL,
+	    (int (*)(void *, char **)) &osip_record_route_to_str },
+	  { "Route: ", 7, NULL, NULL,
+	    (int (*)(void *, char **)) &osip_route_to_str },
+	  { "From: ", 6, NULL, NULL,
+	    (int (*)(void *, char **)) &osip_from_to_str },
+	  { "To: ", 4, NULL, NULL,
+	    (int (*)(void *, char **)) &osip_to_to_str },
+	  { "Call-ID: ", 9, NULL, NULL,
+	    (int (*)(void *, char **)) &osip_call_id_to_str },
+	  { "CSeq: ", 6, NULL, NULL,
+	    (int (*)(void *, char **)) &osip_cseq_to_str },
+	  { "Contact: ", 9, NULL, NULL,
+	    (int (*)(void *, char **)) &osip_contact_to_str },
+	  { "Authorization: ", 15, NULL, NULL,
+	    (int (*)(void *, char **)) &osip_authorization_to_str },
+	  { "WWW-Authenticate: ", 18, NULL, NULL,
+	    (int (*)(void *, char **)) &osip_www_authenticate_to_str },
+	  { "Proxy-Authenticate: ", 20, NULL, NULL,
+	    (int (*)(void *, char **)) &osip_www_authenticate_to_str },
+	  { "Proxy-Authorization: ", 21, NULL, NULL,
+	    (int (*)(void *, char **)) &osip_authorization_to_str },
+	  { "Content-Type: ", 14, NULL, NULL,
+	    (int (*)(void *, char **)) &osip_content_type_to_str },
+	  { "Mime-Version: ", 14, NULL, NULL,
+	    (int (*)(void *, char **)) &osip_content_length_to_str },
+#ifndef MINISIZE
+	  { "Allow: ", 7, NULL, NULL,
+	    (int (*)(void *, char **)) &osip_allow_to_str },
+	  { "Content-Encoding: ", 18, NULL, NULL,
+	    (int (*)(void *, char **)) &osip_content_encoding_to_str },
+	  { "Call-Info: ", 11, NULL, NULL,
+	    (int (*)(void *, char **)) &osip_call_info_to_str },
+	  { "Alert-Info: ", 12, NULL, NULL,
+	    (int (*)(void *, char **)) &osip_call_info_to_str },
+	  { "Error-Info: ", 12, NULL, NULL,
+	    (int (*)(void *, char **)) &osip_call_info_to_str },
+	  { "Accept: ", 8, NULL, NULL,
+	    (int (*)(void *, char **)) &osip_accept_to_str },
+	  { "Accept-Encoding: ", 17, NULL, NULL,
+	    (int (*)(void *, char **)) &osip_accept_encoding_to_str },
+	  { "Accept-Language: ", 17, NULL, NULL,
+	    (int (*)(void *, char **)) &osip_accept_language_to_str },
+	  { "Authentication-Info: ", 21, NULL, NULL,
+	    (int (*)(void *, char **)) &osip_authentication_info_to_str },
+	  { "Proxy-Authentication-Info: ", 27, NULL, NULL,
+	    (int (*)(void *, char **)) &osip_authentication_info_to_str },
+#endif
+	  { { '\0' }, 0, NULL, NULL, NULL }
+	};
+    table[0].header_list = &sip->vias;
+    table[1].header_list = &sip->record_routes;
+    table[2].header_list = &sip->routes;
+    table[3].header_data = sip->from;
+    table[4].header_data = sip->to;
+    table[5].header_data = sip->call_id;
+    table[6].header_data = sip->cseq;
+    table[7].header_list = &sip->contacts;
+    table[8].header_list = &sip->authorizations;
+    table[9].header_list = &sip->www_authenticates;
+    table[10].header_list = &sip->proxy_authenticates;
+    table[11].header_list = &sip->proxy_authorizations;
+    table[12].header_data = sip->content_type;
+    table[13].header_data = sip->mime_version;
+#ifndef MINISIZE
+    table[14].header_list = &sip->allows;
+    table[15].header_list = &sip->content_encodings;
+    table[16].header_list = &sip->call_infos;
+    table[17].header_list = &sip->alert_infos;
+    table[18].header_list = &sip->error_infos;
+    table[19].header_list = &sip->accepts;
+    table[20].header_list = &sip->accept_encodings;
+    table[21].header_list = &sip->accept_languages;
+    table[22].header_list = &sip->authentication_infos;
+    table[23].header_list = &sip->proxy_authentication_infos;
+#endif
 
-  i =
-    strcat_headers_one_per_line (dest, &malloc_size, &message,
-                                 sip->record_routes, "Record-Route: ", 14,
-                                 ((int (*)(void *, char **))
-                                  &osip_record_route_to_str), &next);
-  if (i != 0)
-    {
-      osip_free (*dest);
-      *dest = NULL;
-      return -1;
-    }
-  message = next;
-
-  i =
-    strcat_headers_one_per_line (dest, &malloc_size, &message, sip->routes,
-                                 "Route: ", 7,
-                                 ((int (*)(void *, char **))
-                                  &osip_route_to_str), &next);
-  if (i != 0)
-    {
-      osip_free (*dest);
-      *dest = NULL;
-      return -1;
-    }
-  message = next;
-
-  i = strcat_simple_header (dest, &malloc_size, &message,
-                            sip->from, "From: ", 6,
-                            ((int (*)(void *, char **)) &osip_from_to_str), &next);
-  if (i != 0)
-    {
-      osip_free (*dest);
-      *dest = NULL;
-      return -1;
-    }
-  message = next;
-
-  i = strcat_simple_header (dest, &malloc_size, &message,
-                            sip->to, "To: ", 4,
-                            ((int (*)(void *, char **)) &osip_to_to_str), &next);
-  if (i != 0)
-    {
-      osip_free (*dest);
-      *dest = NULL;
-      return -1;
-    }
-  message = next;
-
-  i = strcat_simple_header (dest, &malloc_size, &message,
-                            sip->call_id, "Call-ID: ", 9,
-                            ((int (*)(void *, char **)) &osip_call_id_to_str),
-                            &next);
-  if (i != 0)
-    {
-      osip_free (*dest);
-      *dest = NULL;
-      return -1;
-    }
-  message = next;
-
-  i = strcat_simple_header (dest, &malloc_size, &message,
-                            sip->cseq, "CSeq: ", 6,
-                            ((int (*)(void *, char **)) &osip_cseq_to_str), &next);
-  if (i != 0)
-    {
-      osip_free (*dest);
-      *dest = NULL;
-      return -1;
-    }
-  message = next;
-
-  i =
-    strcat_headers_one_per_line (dest, &malloc_size, &message, sip->contacts,
-                                 "Contact: ", 9,
-                                 ((int (*)(void *, char **))
-                                  &osip_contact_to_str), &next);
-  if (i != 0)
-    {
-      osip_free (*dest);
-      *dest = NULL;
-      return -1;
-    }
-  message = next;
-
-  i = strcat_headers_one_per_line (dest, &malloc_size, &message,
-                                   sip->authorizations, "Authorization: ", 15,
-                                   ((int (*)(void *, char **))
-                                    &osip_authorization_to_str), &next);
-  if (i != 0)
-    {
-      osip_free (*dest);
-      *dest = NULL;
-      return -1;
-    }
-  message = next;
-
-  i =
-    strcat_headers_one_per_line (dest, &malloc_size, &message,
-                                 sip->www_authenticates, "WWW-Authenticate: ",
-                                 18,
-                                 ((int (*)(void *, char **))
-                                  &osip_www_authenticate_to_str), &next);
-  if (i != 0)
-    {
-      osip_free (*dest);
-      *dest = NULL;
-      return -1;
-    }
-  message = next;
-
-  i =
-    strcat_headers_one_per_line (dest, &malloc_size, &message,
-                                 sip->proxy_authenticates,
-                                 "Proxy-Authenticate: ", 20,
-                                 ((int (*)(void *, char **))
-                                  &osip_www_authenticate_to_str), &next);
-  if (i != 0)
-    {
-      osip_free (*dest);
-      *dest = NULL;
-      return -1;
-    }
-  message = next;
-
-  i =
-    strcat_headers_one_per_line (dest, &malloc_size, &message,
-                                 sip->proxy_authorizations,
-                                 "Proxy-Authorization: ", 21,
-                                 ((int (*)(void *, char **))
-                                  &osip_authorization_to_str), &next);
-  if (i != 0)
-    {
-      osip_free (*dest);
-      *dest = NULL;
-      return -1;
-    }
-  message = next;
+    pos = 0;
+    while (table[pos].header_name[0]!='\0')
+      {
+	if (table[13].header_list==NULL)
+	  i = strcat_simple_header (dest, &malloc_size, &message,
+				    table[pos].header_data,
+				    table[pos].header_name,
+				    table[pos].header_length,
+				    ((int (*)(void *, char **))
+				     table[pos].to_str), &next);
+	i =
+	  strcat_headers_one_per_line (dest, &malloc_size, &message,
+				       table[pos].header_list,
+				       table[pos].header_name,
+				       table[pos].header_length,
+				       ((int (*)(void *, char **))
+					table[pos].to_str), &next);
+	if (i != 0)
+	  {
+	    osip_free (*dest);
+	    *dest = NULL;
+	    return -1;
+	  }
+	message = next;
+	
+	pos++;
+      }
+  }
 
   pos = 0;
-  while (!osip_list_eol (sip->headers, pos))
+  while (!osip_list_eol (&sip->headers, pos))
     {
       osip_header_t *header;
       size_t header_len = 0;
 
-      header = (osip_header_t *) osip_list_get (sip->headers, pos);
+      header = (osip_header_t *) osip_list_get (&sip->headers, pos);
       i = osip_header_to_str (header, &tmp);
       if (i == -1)
         {
@@ -661,8 +636,11 @@ _osip_message_to_str (osip_message_t * sip, char **dest,
       header_len = strlen (tmp);
 
       if (_osip_message_realloc (&message, dest, header_len + 3, &malloc_size) < 0)
-        return -1;
-
+	{
+	  osip_free (tmp);
+          *dest = NULL;
+	  return -1;
+	}
 
       message = osip_str_append (message, tmp);
       osip_free (tmp);
@@ -671,173 +649,13 @@ _osip_message_to_str (osip_message_t * sip, char **dest,
       pos++;
     }
 
-  i =
-    strcat_headers_all_on_one_line (dest, &malloc_size, &message, sip->allows,
-                                    "Allow: ", 7,
-                                    ((int (*)(void *, char **))
-                                     &osip_content_length_to_str), &next);
-  if (i != 0)
-    {
-      osip_free (*dest);
-      *dest = NULL;
-      return -1;
-    }
-  message = next;
-
-  i = strcat_simple_header (dest, &malloc_size, &message,
-                            sip->content_type, "Content-Type: ", 14,
-                            ((int (*)(void *, char **))
-                             &osip_content_type_to_str), &next);
-  if (i != 0)
-    {
-      osip_free (*dest);
-      *dest = NULL;
-      return -1;
-    }
-  message = next;
-
-  i =
-    strcat_headers_all_on_one_line (dest, &malloc_size, &message,
-                                    sip->content_encodings,
-                                    "Content-Encoding: ", 18,
-                                    ((int (*)(void *, char **))
-                                     &osip_content_length_to_str), &next);
-  if (i != 0)
-    {
-      osip_free (*dest);
-      *dest = NULL;
-      return -1;
-    }
-  message = next;
-
-  i = strcat_simple_header (dest, &malloc_size, &message,
-                            sip->mime_version, "Mime-Version: ", 14,
-                            ((int (*)(void *, char **))
-                             &osip_content_length_to_str), &next);
-  if (i != 0)
-    {
-      osip_free (*dest);
-      *dest = NULL;
-      return -1;
-    }
-  message = next;
-
-  i =
-    strcat_headers_one_per_line (dest, &malloc_size, &message,
-                                 sip->call_infos, "Call-Info: ", 11,
-                                 ((int (*)(void *, char **))
-                                  &osip_call_info_to_str), &next);
-  if (i != 0)
-    {
-      osip_free (*dest);
-      *dest = NULL;
-      return -1;
-    }
-  message = next;
-
-  i =
-    strcat_headers_one_per_line (dest, &malloc_size, &message,
-                                 sip->alert_infos, "Alert-Info: ", 12,
-                                 ((int (*)(void *, char **))
-                                  &osip_call_info_to_str), &next);
-  if (i != 0)
-    {
-      osip_free (*dest);
-      *dest = NULL;
-      return -1;
-    }
-  message = next;
-
-  i =
-    strcat_headers_one_per_line (dest, &malloc_size, &message,
-                                 sip->error_infos, "Error-Info: ", 12,
-                                 ((int (*)(void *, char **))
-                                  &osip_call_info_to_str), &next);
-  if (i != 0)
-    {
-      osip_free (*dest);
-      *dest = NULL;
-      return -1;
-    }
-  message = next;
-
-  i =
-    strcat_headers_all_on_one_line (dest, &malloc_size, &message,
-                                    sip->accepts, "Accept: ", 8,
-                                    ((int (*)(void *, char **))
-                                     &osip_accept_to_str), &next);
-  if (i != 0)
-    {
-      osip_free (*dest);
-      *dest = NULL;
-      return -1;
-    }
-  message = next;
-
-  i =
-    strcat_headers_all_on_one_line (dest, &malloc_size, &message,
-                                    sip->accept_encodings,
-                                    "Accept-Encoding: ", 17,
-                                    ((int (*)(void *, char **))
-                                     &osip_accept_encoding_to_str), &next);
-  if (i != 0)
-    {
-      osip_free (*dest);
-      *dest = NULL;
-      return -1;
-    }
-  message = next;
-
-  i =
-    strcat_headers_all_on_one_line (dest, &malloc_size, &message,
-                                    sip->accept_languages,
-                                    "Accept-Language: ", 17,
-                                    ((int (*)(void *, char **))
-                                     &osip_accept_encoding_to_str), &next);
-  if (i != 0)
-    {
-      osip_free (*dest);
-      *dest = NULL;
-      return -1;
-    }
-  message = next;
-
-  i =
-    strcat_headers_one_per_line (dest, &malloc_size, &message,
-                                 sip->authentication_infos,
-                                 "Authentication-Info: ", 21,
-                                 ((int (*)(void *, char **))
-                                  &osip_authentication_info_to_str), &next);
-  if (i != 0)
-    {
-      osip_free (*dest);
-      *dest = NULL;
-      return -1;
-    }
-  message = next;
-
-  i =
-    strcat_headers_one_per_line (dest, &malloc_size, &message,
-                                 sip->proxy_authentication_infos,
-                                 "Proxy-Authentication-Info: ", 27,
-                                 ((int (*)(void *, char **))
-                                  &osip_authentication_info_to_str), &next);
-  if (i != 0)
-    {
-      osip_free (*dest);
-      *dest = NULL;
-      return -1;
-    }
-  message = next;
-
-
   /* we have to create the body before adding the contentlength */
   /* add enough lenght for "Content-Length: " */
 
   if (_osip_message_realloc (&message, dest, 16, &malloc_size) < 0)
     return -1;
 
-  if (sipfrag && osip_list_eol (sip->bodies, 0))
+  if (sipfrag && osip_list_eol (&sip->bodies, 0))
     {
       /* end of headers */
       osip_strncpy (message, CRLF, 2);
@@ -871,7 +689,7 @@ _osip_message_to_str (osip_message_t * sip, char **dest,
      }
      else
      { */
-  if (osip_list_eol (sip->bodies, 0))   /* no body */
+  if (osip_list_eol (&sip->bodies, 0))   /* no body */
     message = osip_strn_append (message, "0", 1);
   else
     {
@@ -897,7 +715,7 @@ _osip_message_to_str (osip_message_t * sip, char **dest,
   start_of_bodies = message;
   total_length = start_of_bodies - *dest;
 
-  if (osip_list_eol (sip->bodies, 0))
+  if (osip_list_eol (&sip->bodies, 0))
     {
       /* same remark as at the beginning of the method */
       sip->message_property = 1;
@@ -916,7 +734,7 @@ _osip_message_to_str (osip_message_t * sip, char **dest,
       osip_generic_param_t *ct_param = NULL;
 
       /* find the boundary */
-      i = osip_generic_param_get_byname (sip->content_type->gen_params,
+      i = osip_generic_param_get_byname (&sip->content_type->gen_params,
                                          "boundary", &ct_param);
       if ((i >= 0) && ct_param && ct_param->gvalue)
         {
@@ -942,12 +760,12 @@ _osip_message_to_str (osip_message_t * sip, char **dest,
     }
 
   pos = 0;
-  while (!osip_list_eol (sip->bodies, pos))
+  while (!osip_list_eol (&sip->bodies, pos))
     {
       osip_body_t *body;
       size_t body_length;
 
-      body = (osip_body_t *) osip_list_get (sip->bodies, pos);
+      body = (osip_body_t *) osip_list_get (&sip->bodies, pos);
 
       if (boundary)
         {
