@@ -1,6 +1,6 @@
 /*
   The oSIP library implements the Session Initiation Protocol (SIP -rfc3261-)
-  Copyright (C) 2001,2002,2003,2004,2005  Aymeric MOIZARD jack@atosc.org
+  Copyright (C) 2001,2002,2003,2004,2005,2006,2007 Aymeric MOIZARD jack@atosc.org
   
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -54,6 +54,7 @@ osip_message_set_proxy_authorization (osip_message_t * sip, const char *hvalue)
   return 0;
 }
 
+#ifndef MINISIZE
 int
 osip_message_get_proxy_authorization (const osip_message_t * sip, int pos,
                                       osip_proxy_authorization_t ** dest)
@@ -68,3 +69,4 @@ osip_message_get_proxy_authorization (const osip_message_t * sip, int pos,
   *dest = proxy_authorization;
   return pos;
 }
+#endif
