@@ -1,6 +1,6 @@
 /*
   The oSIP library implements the Session Initiation Protocol (SIP -rfc3261-)
-  Copyright (C) 2001,2002,2003,2004,2005  Aymeric MOIZARD jack@atosc.org
+  Copyright (C) 2001,2002,2003,2004,2005,2006,2007 Aymeric MOIZARD jack@atosc.org
   
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -24,11 +24,13 @@
 #include <osipparser2/osip_message.h>
 #include <osipparser2/osip_parser.h>
 
+#ifndef MINISIZE
 int
 osip_record_route_init (osip_record_route_t ** record_route)
 {
   return osip_from_init ((osip_from_t **) record_route);
 }
+#endif
 
 /* adds the record_route header to message.         */
 /* INPUT : const char *hvalue | value of header.    */
@@ -57,6 +59,7 @@ osip_message_set_record_route (osip_message_t * sip, const char *hvalue)
   return 0;
 }
 
+#ifndef MINISIZE
 /* returns the record_route header.    */
 /* INPUT : osip_message_t *sip | sip message.   */
 /* returns null on error. */
@@ -73,7 +76,9 @@ osip_message_get_record_route (const osip_message_t * sip, int pos,
   *dest = record_route;
   return pos;
 }
+#endif
 
+#ifndef MINISIZE
 int
 osip_record_route_parse (osip_record_route_t * record_route, const char *hvalue)
 {
@@ -155,3 +160,5 @@ osip_record_route_free (osip_record_route_t * record_route)
 {
   osip_from_free ((osip_from_t *) record_route);
 }
+
+#endif
