@@ -1,6 +1,6 @@
 /*
   The oSIP library implements the Session Initiation Protocol (SIP -rfc3261-)
-  Copyright (C) 2001,2002,2003,2004,2005  Aymeric MOIZARD jack@atosc.org
+  Copyright (C) 2001,2002,2003,2004,2005,2006,2007 Aymeric MOIZARD jack@atosc.org
   
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -38,24 +38,9 @@ osip_uri_init (osip_uri_t ** url)
   (*url)->host = NULL;
   (*url)->port = NULL;
 
-  (*url)->url_params = (osip_list_t *) osip_malloc (sizeof (osip_list_t));
-  if ((*url)->url_params == NULL)
-    {
-      osip_free (*url);
-      *url = NULL;
-      return -1;
-    }
-  osip_list_init ((*url)->url_params);
+  osip_list_init (&(*url)->url_params);
 
-  (*url)->url_headers = (osip_list_t *) osip_malloc (sizeof (osip_list_t));
-  if ((*url)->url_headers == NULL)
-    {
-      osip_free ((*url)->url_params);
-      osip_free (*url);
-      *url = NULL;
-      return -1;
-    }
-  osip_list_init ((*url)->url_headers);
+  osip_list_init (&(*url)->url_headers);
 
   (*url)->string = NULL;
   return 0;
@@ -603,12 +588,12 @@ osip_uri_to_str (const osip_uri_t * url, char **dest)
     int pos = 0;
     osip_uri_param_t *u_param;
 
-    while (!osip_list_eol (url->url_params, pos))
+    while (!osip_list_eol (&url->url_params, pos))
       {
         char *tmp1;
         char *tmp2 = NULL;
 
-        u_param = (osip_uri_param_t *) osip_list_get (url->url_params, pos);
+        u_param = (osip_uri_param_t *) osip_list_get (&url->url_params, pos);
 
         tmp1 = __osip_uri_escape_uri_param (u_param->gname);
         if (u_param->gvalue == NULL)
@@ -638,17 +623,25 @@ osip_uri_to_str (const osip_uri_t * url, char **dest)
     int pos = 0;
     osip_uri_header_t *u_header;
 
-    while (!osip_list_eol (url->url_headers, pos))
+    while (!osip_list_eol (&url->url_headers, pos))
       {
         char *tmp1;
         char *tmp2;
 
-        u_header = (osip_uri_header_t *) osip_list_get (url->url_headers, pos);
+        u_header = (osip_uri_header_t *) osip_list_get (&url->url_headers, pos);
         tmp1 = __osip_uri_escape_header_param (u_header->gname);
+
+        if (tmp1 == NULL)
+          {
+            osip_free (buf);
+            return -1;
+          }
+
         tmp2 = __osip_uri_escape_header_param (u_header->gvalue);
 
-        if (tmp1 == NULL || tmp2 == NULL)
+        if (tmp2 == NULL)
           {
+            osip_free (tmp1);
             osip_free (buf);
             return -1;
           }
@@ -686,18 +679,17 @@ osip_uri_free (osip_uri_t * url)
   osip_free (url->host);
   osip_free (url->port);
 
-  osip_uri_param_freelist (url->url_params);
+  osip_uri_param_freelist (&url->url_params);
 
   {
     osip_uri_header_t *u_header;
 
-    while (!osip_list_eol (url->url_headers, pos))
+    while (!osip_list_eol (&url->url_headers, pos))
       {
-        u_header = (osip_uri_header_t *) osip_list_get (url->url_headers, pos);
-        osip_list_remove (url->url_headers, pos);
+        u_header = (osip_uri_header_t *) osip_list_get (&url->url_headers, pos);
+        osip_list_remove (&url->url_headers, pos);
         osip_uri_header_free (u_header);
       }
-    osip_free (url->url_headers);
   }
 
   osip_free (url->string);
@@ -733,37 +725,18 @@ osip_uri_clone (const osip_uri_t * url, osip_uri_t ** dest)
   if (url->string != NULL)
     ur->string = osip_strdup (url->string);
 
-  {
-    int pos = 0;
-    osip_uri_param_t *u_param;
-    osip_uri_param_t *dest_param;
-
-    while (!osip_list_eol (url->url_params, pos))
-      {
-        u_param = (osip_uri_param_t *) osip_list_get (url->url_params, pos);
-        i = osip_uri_param_clone (u_param, &dest_param);
-        if (i != 0)
-          return -1;
-        osip_list_add (ur->url_params, dest_param, -1);
-        pos++;
-      }
-  }
-  {
-    int pos = 0;
-    osip_uri_param_t *u_param;
-    osip_uri_param_t *dest_param;
-
-    while (!osip_list_eol (url->url_headers, pos))
-      {
-        u_param = (osip_uri_param_t *) osip_list_get (url->url_headers, pos);
-        i = osip_uri_param_clone (u_param, &dest_param);
-        if (i != 0)
-          return -1;
-        osip_list_add (ur->url_headers, dest_param, -1);
-        pos++;
-      }
-  }
-
+  i = osip_list_clone(&url->url_params, &ur->url_params, (int *(*)(void *, void *)) &osip_uri_param_clone);
+  if (i != 0)
+    {
+      osip_uri_free(ur);
+      return -1;
+    }
+  i = osip_list_clone(&url->url_headers, &ur->url_headers, (int *(*)(void *, void *)) &osip_uri_param_clone);
+  if (i != 0)
+    {
+      osip_uri_free(ur);
+      return -1;
+    }
   *dest = ur;
   return 0;
 }
@@ -827,7 +800,6 @@ osip_uri_param_freelist (osip_list_t * params)
       osip_list_remove (params, 0);
       osip_uri_param_free (u_param);
     }
-  osip_free (params);
 }
 
 int
@@ -999,7 +971,7 @@ __osip_uri_unescape (char *string)
       if ('%' == in)
         {
           /* encoded part */
-          if (sscanf (ptr + 1, "%02X", &hex) == 1)
+          if (alloc > 2 && sscanf (ptr + 1, "%02X", &hex) == 1)
             {
               in = (unsigned char) hex;
               if (*(ptr + 2) &&
@@ -1015,6 +987,10 @@ __osip_uri_unescape (char *string)
                   ptr += 1;
                 }
             }
+          else
+            {
+              break;
+            }
         }
 
       string[index++] = in;
