@@ -1,6 +1,6 @@
 /*
   The oSIP library implements the Session Initiation Protocol (SIP -rfc3261-)
-  Copyright (C) 2001,2002,2003,2004,2005  Aymeric MOIZARD jack@atosc.org
+  Copyright (C) 2001,2002,2003,2004,2005,2006,2007 Aymeric MOIZARD jack@atosc.org
   
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -76,6 +76,7 @@ osip_message_append_via (osip_message_t * sip, const char *hvalue)
   return 0;
 }
 
+#ifndef MINISIZE
 /* returns the via header.                         */
 /* INPUT : int pos | pos of via header.            */
 /* INPUT : osip_message_t *sip | sip message.               */
@@ -93,7 +94,7 @@ osip_message_get_via (const osip_message_t * sip, int pos, osip_via_t ** dest)
 
   return pos;
 }
-
+#endif
 
 int
 osip_via_init (osip_via_t ** via)
@@ -447,24 +448,12 @@ osip_via_clone (const osip_via_t * via, osip_via_t ** dest)
   if (via->comment != NULL)
     vi->comment = osip_strdup (via->comment);
 
-  {
-    int pos = 0;
-    osip_generic_param_t *u_param;
-    osip_generic_param_t *dest_param;
-
-    while (!osip_list_eol (&via->via_params, pos))
-      {
-        u_param = (osip_generic_param_t *) osip_list_get (&via->via_params, pos);
-        i = osip_generic_param_clone (u_param, &dest_param);
-        if (i != 0)
-          {
-            osip_via_free (vi);
-            return -1;
-          }
-        osip_list_add (&vi->via_params, dest_param, -1);
-        pos++;
-      }
-  }
+  i = osip_list_clone(&via->via_params, &vi->via_params, (int *(*)(void *, void *)) &osip_generic_param_clone);
+  if (i != 0)
+    {
+      osip_via_free(vi);
+      return -1;
+    }
   *dest = vi;
   return 0;
 }
