@@ -1,6 +1,6 @@
 /*
   The oSIP library implements the Session Initiation Protocol (SIP -rfc3261-)
-  Copyright (C) 2001,2002,2003,2004,2005  Aymeric MOIZARD jack@atosc.org
+  Copyright (C) 2001,2002,2003,2004,2005,2006,2007 Aymeric MOIZARD jack@atosc.org
   
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -23,35 +23,38 @@
 
 #ifndef DOXYGEN
 
+#ifndef MINISIZE
 #define NUMBER_OF_HEADERS 33
+#else
+#define NUMBER_OF_HEADERS 22
+#endif
 
 /* internal type for parser's config */
 typedef struct ___osip_message_config_t
 {
   char *hname;
   int (*setheader) (osip_message_t *, const char *);
+  int ignored_when_invalid;
 }
 __osip_message_config_t;
 
-int __osip_message_call_method (int i, osip_message_t * dest,
-				const char *hvalue);
+int __osip_message_call_method (int i, osip_message_t * dest, const char *hvalue);
 int __osip_message_is_known_header (const char *hname);
 
 int __osip_find_next_occurence (const char *str, const char *buf,
-				const char **index_of_str);
+                                const char **index_of_str, const char *end_of_buf);
 int __osip_find_next_crlf (const char *start_of_header,
-			   const char **end_of_header);
+                           const char **end_of_header);
 int __osip_find_next_crlfcrlf (const char *start_of_part,
-			       const char **end_of_part);
+                               const char **end_of_part);
 
 int __osip_quoted_string_set (const char *name, const char *str,
-			      char **result, const char **next);
+                              char **result, const char **next);
 int __osip_token_set (const char *name, const char *str,
-		      char **result, const char **next);
+                      char **result, const char **next);
 
 
-int __osip_generic_param_parseall (osip_list_t * gen_params,
-				   const char *params);
+int __osip_generic_param_parseall (osip_list_t * gen_params, const char *params);
 #endif
 
 #endif
